const Aws = require('aws-sdk');
const https = require('https');
const url = require('url');

exports.handler = function (event, context, callback) {
    /* Example request
        {
            "RequestType": "Create",
            "ResponseURL": "http://pre-signed-S3-url-for-response",
            "StackId": "arn:aws:cloudformation:us-west-2:EXAMPLE/stack-name/guid",
            "RequestId": "unique-id-for-this-create-request",
            "ResourceType": "Custom::TestResource",
            "LogicalResourceId": "MyTestResource",
            "ResourceProperties": {
                "Name": "Value",
                "List": [ "1", "2", "3" ]
            }
        }
    */
    console.info('Received request:', JSON.stringify(event, null, 4));
    let customDomainName = event.ResourceProperties.CustomDomainName;
    if (event.RequestType === 'Delete') {
        sendResponse(event, context.logStreamName, false, undefined, callback);
    } else {
        const apigw = new Aws.APIGateway();
        apigw.getDomainName({ domainName: customDomainName }, (error, data) => {
            sendResponse(event, context.logStreamName, error,
                error ?
                    { Error: 'getDomainName call failed:', error } :
                    { RegionalDomainName: data.regionalDomainName, RegionalZoneId: data.regionalHostedZoneId },
                callback
            )
        })
    }
};

function sendResponse(event, logStreamName, error, responseData, callback) {
    /* Example response
        {
            "Status": "SUCCESS",
            "Reason": "See the details in CloudWatch Log Stream: 2017/08/15/[$LATEST]d34570dd73364f4788a90381e578668a",
            "PhysicalResourceId": "2017/08/15/[$LATEST]d34570dd73364f4788a90381e578668a",
            "StackId": "arn:aws:cloudformation:us-west-2:EXAMPLE/stack-name/guid",
            "RequestId": "unique-id-for-this-create-request",
            "LogicalResourceId": "MyTestResource",
            "Data": {
                "IotEndpointAddress": "ajuxuvth6k0hg.iot.us-east-1.amazonaws.com"
            }
        }
    */
    const responseBody = JSON.stringify({
        Status: error ? 'FAILED' : 'SUCCESS',
        Reason: `See the details in CloudWatch Log Stream: ${logStreamName}`,
        Error: error,
        PhysicalResourceId: logStreamName,
        StackId: event.StackId,
        RequestId: event.RequestId,
        LogicalResourceId: event.LogicalResourceId,
        Data: responseData
    }, null, 4);
    console.info('Response body:', responseBody);

    const parsedUrl = url.parse(event.ResponseURL);
    const request = https.request({
        protocol: parsedUrl.protocol,
        hostname: parsedUrl.hostname,
        port: 443,
        path: parsedUrl.path,
        method: 'PUT',
        headers: {
            'Content-Length': responseBody.length,
        }
    }, response => {
        const message = `Callback response:  ${response.statusCode}-${response.statusMessage}-${response.body}`;
        console.info(message);
        callback(null, message);
    });

    request.on('error', error => {
        const message = `Callback response: ${error}`;
        console.error(message);
        callback(message, null);
    });

    request.write(responseBody);
    request.end();
}
